# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####
'''
Bevel add-on
'''
#--- ### Header
bl_info = {
    "name": "Bevel",
    "author": "Witold Jaworski",
    "version": (1, 0, 1),
    "blender": (2, 5, 7),
    "api": 36147,
    "location": "View3D >Specials (W-key)",
    "category": "Mesh",
    "description": "Bevels selected edges",
    "warning": "",
    "wiki_url": "http://airplanes3d.net/scripts-252_e.xml",
    "tracker_url": "http://airplanes3d.net/track-252_e.xml"
    }
#--- ### Imports
import bpy
from bpy.utils import register_module, unregister_module
from bpy.props import FloatProperty
#--- ### Core operation
def bevel(obj, width):
    """Bevels selected edges of the mesh
       Arguments:
            @obj (Object): an object with a mesh. 
                           It should have some edges selected
            @width (float):width of the bevel
       This function should be called in the Edit Mode, only!
    """    
    #
    #edge = bpy.types.MeshEdge
    #obj = bpy.types.Object
    #bevel = bpy.types.BevelModifier

    bpy.ops.object.editmode_toggle() #switch into OBJECT mode
    #adding the Bevel modifier
    bpy.ops.object.modifier_add(type = 'BEVEL')  
    bevel = obj.modifiers[-1] #the new modifier is always added at the end
    bevel.limit_method = 'WEIGHT'
    bevel.edge_weight_method = 'LARGEST'
    bevel.width = width
    #moving it up, to the first position on the modifier stack:
    while obj.modifiers[0] != bevel:
        bpy.ops.object.modifier_move_up(modifier = bevel.name)
        
    for edge in obj.data.edges:
        if edge.select:
            edge.bevel_weight = 1.0 
    
    bpy.ops.object.modifier_apply(apply_as = 'DATA', modifier = bevel.name)
    
    #clean up after applying our modifier: remove bevel weights:
    for edge in obj.data.edges:
        if edge.select:
            edge.bevel_weight = 0.0 
            
    bpy.ops.object.editmode_toggle() #switch back into EDIT_MESH mode

#--- ### Operator
class Bevel(bpy.types.Operator):
    ''' Bevels selected edges of the mesh'''
    bl_idname = "mesh.mbevel"
    bl_label = "Bevel"
    bl_description = "Bevels selected edges"
    bl_options = {'REGISTER', 'UNDO'} #Set this options, if you want to update  
    #                                  parameters of this operator interactively 
    #                                  (in the Tools pane) 
    #--- parameters
    width = FloatProperty(name="Width", description="Bevel width", 
                          subtype = 'DISTANCE', default = 0.1, min = 0.0, 
                                                    step = 1, precision = 2)
    #--- other fields
    LAST_WIDTH_NAME = "mesh.bevel.last_width" #the name of the custom scene property 
    
    #--- Blender interface methods
    @classmethod
    def poll(cls,context):
        return (context.mode == 'EDIT_MESH')

    def invoke(self, context, event):
        #input validation: are there any edges selected?
        bpy.ops.object.editmode_toggle()
        selected = list(filter(lambda e: e.select, context.object.data.edges))
        bpy.ops.object.editmode_toggle()
            
        if len(selected) > 0:
            last_width = context.scene.get(self.LAST_WIDTH_NAME,None)
            if last_width: 
                self.width = last_width
            return self.execute(context)
        else:
            self.report(type='ERROR', message="No edges selected")
            return {'CANCELLED'}
        
    def execute(self,context):
        bevel(context.object,self.width)
        context.scene[self.LAST_WIDTH_NAME] = self.width
        return {'FINISHED'}

def menu_draw(self, context):
    self.layout.operator_context = 'INVOKE_REGION_WIN'
    self.layout.operator(Bevel.bl_idname, "Bevel")
    
#--- ### Register
def register():
    register_module(__name__)
    bpy.types.VIEW3D_MT_edit_mesh_specials.prepend(menu_draw)
    
def unregister():
    bpy.types.VIEW3D_MT_edit_mesh_specials.remove(menu_draw)
    unregister_module(__name__)
    
#--- ### Main code    
if __name__ == '__main__':
    register()

